@@ -7,14 +7,16 @@ import json
 import sys
 
 import requests
+from requests.auth import HTTPBasicAuth
 import yaml
 import time
 
-VERSION = '0.1.3'
+VERSION = '0.1.4'
 
 default_backliftURL = 'http://backlift.com/'
 backlift_configfile = '.backlift'
 backlift_email = 'support@backlift.com'
+backlift_dir = '.backlift'
 
 # maxfiles is set to a few less than the system limit. 
 maxfiles = resource.getrlimit(resource.RLIMIT_NOFILE)[0] - 6
@@ -48,8 +50,8 @@ ERR_500 =      "Ooops! Our server is having trouble. We're looking into\n"+\
 ERR_404 =      "Ooops! We couldn't find the resource at %s."
 
 ERR_403 =      "Ooops! This action is forbidden. Either you haven't\n" +\
-               "registered your early access code, or you used an\n"+\
-               "invalid code. Please see backlift --help."
+               "set up your api key, or you used an invalid key. Please\n"+\
+               "use the backlift setup command. See backlift --help."
 
 ERR_RESPONSE = "Oops! Something is wrong with the server. Please send an\n"+\
                "angry email to %s." % backlift_email
@@ -63,18 +65,16 @@ def fail(msg):
 
 
 def check_response(r):
+    # import pdb; pdb.set_trace()
     if not (r.status_code / 100 == 2):
-        if r.text:
-            mtype = r.headers['Content-Type'].split(';')[0]
-            if r.status_code == 500:
-                fail(ERR_500)
-            elif r.status_code == 404:
-                fail(ERR_404 % r.request.url)
-            elif r.status_code == 403:
-                fail(ERR_403)
-            else:
-                fail(ERR_RESPONSE)
-        else: 
+        mtype = r.headers['Content-Type'].split(';')[0]
+        if r.status_code == 500:
+            fail(ERR_500)
+        elif r.status_code == 404:
+            fail(ERR_404 % r.request.url)
+        elif r.status_code == 403:
+            fail(ERR_403)
+        else:
             fail(ERR_RESPONSE)
 
 
@@ -147,7 +147,52 @@ def get_id(configfile):
     return id
 
 
+def open_create(path, perm):
+    try:
+        newfile = open(path, perm)
+    except IOError:
+        newfile = None
 
+    if not newfile:
+        try:
+            filedir = os.path.split(path)[0]
+            os.makedirs(filedir, 0755)
+            newfile = open(path, perm)
+        except OSError:
+            fail(ERR_WRITE % filedir)
+
+    return newfile
+
+
+def get_api_key_file():
+    home = os.getenv('BACKLIFT_HOME') or os.path.expanduser('~')
+    path = os.path.join(home, backlift_dir, 'api_key')
+    return path
+
+
+def save_api_key(api_key):
+    fh = open_create(get_api_key_file(), 'w')
+    fh.write(api_key)
+    fh.close()
+
+
+STATIC_API_KEY = None
+
+def read_api_key():
+    global STATIC_API_KEY
+
+    if STATIC_API_KEY:
+        return STATIC_API_KEY
+
+    try:
+        fh = open(get_api_key_file(), 'r')
+        api_key = fh.read()
+        fh.close()
+    except IOError:
+        api_key = ''
+
+    STATIC_API_KEY = api_key
+    return api_key
 
 
 ## REQUESTS -------------------------------------------------------------------
@@ -161,10 +206,11 @@ def register_key(key):
         fail(ERR_SERVER)
 
 
-def create_app():
+def create_app(api_key):
     try:
         # import pdb; pdb.set_trace()
-        r = requests.post(backliftURL_createapp)
+        r = requests.post(backliftURL_createapp, 
+                          auth=HTTPBasicAuth('api', api_key))
         check_response(r)
 
         r = json.loads(r.text)
@@ -176,13 +222,13 @@ def create_app():
     return None
 
 
-def download_template_files(path, template, params=None):
+def download_template_files(path, template, api_key, params=None):
     files = []
     try:
         # import pdb; pdb.set_trace()
 
         manifest_url = os.path.join(backliftURL_templatefiles, template)
-        r = requests.get(manifest_url)
+        r = requests.get(manifest_url, auth=HTTPBasicAuth('api', api_key))
         check_response(r)
 
         r = json.loads(r.text)
@@ -192,24 +238,13 @@ def download_template_files(path, template, params=None):
             for f in files:
 
                 file_url = os.path.join(backliftURL_templatefiles, template, f)
-                r = requests.get(file_url, params=params)
+                r = requests.get(file_url, params=params, 
+                                 auth=HTTPBasicAuth('api', api_key))
                 check_response(r)
 
                 filepath = os.path.abspath(os.path.join(path, f))
 
-                try:
-                    newfile = open(filepath, 'wb')
-                except IOError:
-                    newfile = None
-
-                if not newfile:
-                    try:
-                        filedir = os.path.split(filepath)[0]
-                        os.makedirs(filedir, 0755)
-                        newfile = open(filepath, 'wb')
-                    except OSError:
-                        fail(ERR_WRITE % filedir)
-
+                newfile = open_create(filepath, 'wb')
                 newfile.write(r.content)
                 newfile.close()
                 print "creating %s" % os.path.join(path, f)
@@ -220,9 +255,10 @@ def download_template_files(path, template, params=None):
     return None
 
 
-def upload_files(id, files):
+def upload_files(id, files, api_key):
     try:
-        r = requests.put(backliftURL_upload % id, files=files)
+        r = requests.put(backliftURL_upload % id, files=files,
+                         auth=HTTPBasicAuth('api', api_key))
 
         for filekey, filedata in files.items():
             filedata[1].close()
@@ -241,6 +277,11 @@ def upload_files(id, files):
 
 ## COMMANDS -------------------------------------------------------------------
 
+
+def setup(api_key):
+    save_api_key(api_key)
+
+
 def register(key):
     register_key(key)
 
@@ -249,6 +290,8 @@ def create(path, name='', template='blank'):
 
     # import pdb; pdb.set_trace()
 
+    api_key = read_api_key()
+
     app_root = os.path.normpath(
                     os.path.abspath(
                         os.path.join(path, name)))
@@ -258,9 +301,9 @@ def create(path, name='', template='blank'):
     if (os.path.exists(cfg_path)):
         fail(ERR_INIT)
  
-    id = create_app()
+    id = create_app(api_key)
     if id:
-        download_template_files(app_root, template, {
+        download_template_files(app_root, template, api_key, {
             'app_id': id
         })
 
@@ -273,25 +316,31 @@ def create(path, name='', template='blank'):
 
 
 def init(path):
-    create(path)
+    api_key = read_api_key()
+    create(path, api_key)
 
 
 def push(path, skip_hidden):
 
+    api_key = read_api_key()
+
     # scan path for files to upload 
 
     files, configfile = collectFiles(path, skip_hidden)
 
     id = get_id(configfile)
 
-    upload_files(id, files)
+    upload_files(id, files, api_key)
 
     return id
 
 
 def watch(path, skip_hidden):
+
     # import pdb; pdb.set_trace()
 
+    api_key = read_api_key()
+
     id = push(path, skip_hidden)
 
     before, configfile = scanFiles(path, skip_hidden)
@@ -311,7 +360,7 @@ def watch(path, skip_hidden):
             
             if added or removed or modified:    
                 files, configfile = collectFiles(path, skip_hidden)        
-                upload_files(id, files)
+                upload_files(id, files, api_key)
     
             before = after
 
@@ -333,7 +382,7 @@ def execute_command_line():
 
     usagetxt = '%prog COMMAND [options]\n\n' +\
         'Commands:\n' +\
-        '  register KEY\t\tRegister this computer for early access\n' +\
+        '  setup API_KEY\t\tAuthorize this computer with your API_KEY\n' +\
         '  create NAME\t\tCreate a new basic backlift app in the NAME folder\n' +\
         '  create:TYPE NAME\tCreate a new backlift app using the TYPE template\n' +\
         '  init\t\t\tInitialize backlift for an existing app\n' +\
@@ -362,11 +411,11 @@ def execute_command_line():
     if len(args) < 1:
         parser.print_help()
     else:
-        if   args[0].lower() == 'register':
+        if   args[0].lower() == 'setup':
             if len(args) != 2:
                 parser.print_help()
             else:
-                register(args[1])
+                setup(args[1])
         elif args[0].lower().startswith('create'):
             if len(args) != 2:
                 parser.print_help()
